// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <cassert>

#include "OdfDesignators.h"

int main()
{
	//	OdfSet
	assert(OdfSet::isValid("0335_0068940601_R2S92107DII.FIT"));
	std::string name("/orbit/flight_data/ODF/0335/0068940601/0335_0068940601_R2S92107DII.FIT:TABLE%ATT");
	assert(OdfSet::isValid(name));
	OdfSet odfSet(name);
	assert(odfSet.dirname() == "/orbit/flight_data/ODF/0335/0068940601");
	assert(odfSet.basename() == "0335_0068940601_R2S92107DII.FIT");
	assert(odfSet.revolutionNr() == 335);
	assert(odfSet.exposureNr() == 921);
	assert(odfSet.scheduled());

	assert(!OdfSet::isValid("bla.txt"));

	name = "/users/gvacanti/tmp/odfingest/earlydata/0122320401/0059_0122320401_M1X00000PEH.FIT";
	OdfSet xxx(name);
	std::cout<<xxx.basename()<<' '<<xxx.dirname()<<std::endl;

	//	InstrumentExposureDesignator
	std::string s("R2S921");
	InstrumentExposureIdentifier i1(odfSet.basename()),
								 i2(s);
	assert(i1.instrument() == i2.instrument() && i1.instrument() == Xmm::RGS2);
	assert(i1.scheduleFlagApplicable() && i2.scheduleFlagApplicable());
	assert(i1.scheduled() && i2.scheduled());
	assert(i1.exposureNumber() == i2.exposureNumber() &&
		   i1.exposureNumber() == 921);

	assert(i1.instrumentExposureId() == i2.instrumentExposureId() &&
		   i1.instrumentExposureId() == s);
	assert(i1.exposureId() == "S921");

	std::cout<<"\nAll tests successfully passed.\n";

	return 0;
}
